//
//  CpCdViewModel.swift
//  Product Calculator
//
//  Created by Suraj Kumar Mandal on 30/11/21.
//

import Foundation
import Alamofire

class CpCdViewModel {
    
    var delegate: CpCdViewProtocol?
    
    var bankCpCdModel = [CpCdModel]()
    
    //POST Bank CP/CD API
    func getCpCdOutput(depositAmount:String, annualInterest:String, tenure:String, compoundFrequency:String, depositDate:String) {
        if let delegate = delegate {
            delegate.startLoader()
            let url = ApiUrl.BASE_URL + ApiUrl.GET_BANK_CPCD
            
            var compound = String()
            switch compoundFrequency {
            case "Monthly":
                compound = "12"
            case "Bi-Monthly":
                compound = "6"
            case "Quarterly":
                compound = "4"
            case "Triannually":
                compound = "3"
            case "Half Yearly":
                compound = "2"
            case "Annually":
                compound = "1"
            default:
                compound = ""
            }
                        
            let parameters: [String: String] = [
                "depositAmount": depositAmount,
                "annualInterest": annualInterest,
                "termYearsDays": "D",
                "tenure": tenure,
                "compoundFrequency": compound,
                "depositDate": depositDate
            ]
            print(parameters)
            
            AF.request(url, method: .post, parameters: parameters, encoding: JSONEncoding.default).responseJSON { response in
                
                switch response.result {
                case .success(let value):
                    delegate.stopLoader()
                    print(value)
                    let data = value as! NSDictionary
                    print(data)
                    DispatchQueue.main.async {
                        let depositAmount = data["depositAmount"] as! Double
                        let annualInterest = data["annualInterest"] as! Double
                        let term = data["term"] as! Int
                        let compundFreq = data["compundFreq"] as! Double
                        let depositDate = data["depositDate"] as? String
                        let maturityAmount = data["maturityAmount"] as! Double
                        let interestReceived = data["interestReceived"] as! Double
                        let totalInterestReceived = data["totalInterestReceived"] as! Double
                        let maturityDate = data["maturityDate"] as! String
                        
                        let bankCpCdLookupList = data["bankFdTdrLookupList"] as! [NSDictionary]
                        
                        let cpcdData = CpCdModel(depositDate: depositDate ?? "", maturityDate: maturityDate, term: term, depositAmount: depositAmount, annualInterest: annualInterest, compundFreq: compundFreq, maturityAmount: maturityAmount, totalInterestReceived: totalInterestReceived, interestReceived: interestReceived)
                        
                        for list in bankCpCdLookupList {
                            let serialNo = list.value(forKey: "serialNo") as! NSNumber
                            let referenceDate = list.value(forKey: "referenceDate") as! String
                            let referenceMonth = list.value(forKey: "referenceMonth") as! String
                            let financialYear = list.value(forKey: "financialYear") as! String
                            let openingBal = list.value(forKey: "openingBal") as! Double
                            let amountDeposited = list.value(forKey: "amountDeposited") as! Double
                            let interestAccrued = list.value(forKey: "interestAccrued") as! Double
                            let interestCredited = list.value(forKey: "interestCredited") as! Double
                            let totalInterestReceived = list.value(forKey: "totalInterestReceived") as! Double
                            let closingBalance = list.value(forKey: "closingBalance") as! Double
                            let daysToMaturity = list.value(forKey: "daysToMaturity") as! NSNumber
                                                        
                            let list = CpCdLookupList(serialNo: Int(truncating: serialNo), openingBal: openingBal, amountDeposited: amountDeposited, interestAccrued: interestAccrued, totalInterestReceived: totalInterestReceived, interestCredited: interestCredited, daysToMaturity: Int(truncating: daysToMaturity), referenceDate: referenceDate, referenceMonth: referenceMonth, financialYear: financialYear, closingBalance: closingBalance)
                            
                            cpcdData.bankCpCdLookupList.append(list)
                        }
                        self.bankCpCdModel.append(cpcdData)
                        delegate.navigate(cpcdData)
                    }
                    
                case .failure(let error):
                    delegate.stopLoader()
                    print(error)
                    //delegate.showError(error: error)
                }
            }
        }
    }
    
}

protocol CpCdViewProtocol {
    func startLoader()
    func stopLoader()
    func showError(error:String)
    func navigate(_ data:CpCdModel)
}
